/* ****** ACTION MENU ****** */

GtkWindow {
	background-color: mix(white, black, 0.1);
}

#menu {
	background-color: mix(white, black, 0.2);
	border-color: mix(white, black, 0.5);
	border-width: 0 0 0 6px;
	padding: 30px;
}

.dark#menu {
	background-color: mix(white, black, 0.8);
	border-width: 0 0 0 6px;
}
