#page1_1080 {
    background-image: url("1080/background1.jpg");
}

#page2_1080 {
    background-image: url("1080/background2.jpg");
}

#page1_720 {
    background-image: url("720/background1.jpg");
}

#page2_720 {
    background-image: url("720/background2.jpg");
}
