/* Copyright 2013 Endless Mobile, Inc. */

#ifndef EOS_INIT_PRIVATE_H
#define EOS_INIT_PRIVATE_H

#include <glib.h>

G_BEGIN_DECLS

gboolean eos_is_inited (void);

G_END_DECLS

#endif /* EOS_INIT_PRIVATE_H */
