/* Copyright 2013 Endless Mobile, Inc. */

#ifndef EOS_TYPES_H
#define EOS_TYPES_H

#if !(defined(_EOS_SDK_INSIDE_ENDLESS_H) || defined(COMPILING_EOS_SDK))
#error "Please do not include this header file directly."
#endif

#include "eosapiversion.h"
#include "eosenums.h"
#include "eosmacros.h"

#include <glib-object.h>

/* Shared typedefs for structures */

#endif /* EOS_TYPES_H */
