/* Wipe all theming CSS to start with. We'll leave the adwaita theme for
 * spinner.*/
.eos-window-inner :not(spinner):not(decoration),
.eos-reset,
.eos-reset :not(spinner):not(decoration) {
    all: unset;
}

EosWindow.in-resize {
    -eos-cairo-filter: -eos-nearest;
}

EosWindow .titlebar .NavigationSearchBox {
    margin-top: 0;
    margin-bottom: 0;
}

.credits-button {
    opacity: 0.0;
    transition: opacity 250ms;
}

.credits-button:hover,
.credits-button:active {
    opacity: 1.0;
}

/*
 * Themeing for context menus, to get Adwaita look
 * FIXME: Really gross to be recreating all this Adwaita theme here.
 * Unfortunately even the :not pseudo-selector won't allow you to select all
 * items whose ancestor isn't a menuitem, because if one ancestor is a
 * menuitem then another ancestor is likely not. So at least some of the below
 * rules will always be necessary.
 */
@define-color endless_menu_fg_color #2e3436;
@define-color endless_menu_bg_color shade (#ededed, 1.1);

.eos-window-inner .context-menu {
    font: initial;
    color: @endless_menu_fg_color;
    background-color: @endless_menu_bg_color;
    padding: 2px 0px;
}

.eos-window-inner .context-menu menuitem {
    padding: 4px;
}

.eos-window-inner .context-menu menuitem arrow {
    min-width: 8px;
    min-height: 8px;
}

.eos-window-inner .context-menu menuitem:active,
.eos-window-inner .context-menu menuitem:hover {
    color: #ffffff;
    background-color: #4a90d9;
}

.eos-window-inner .context-menu menuitem *:disabled {
    color: mix (@endless_menu_fg_color, @endless_menu_bg_color, 0.6);
}

.eos-window-inner .context-menu separator {
    background-color: transparent;
    background-image: image(mix (@endless_menu_fg_color, @endless_menu_bg_color, 0.9));
    background-size: 1px 1px;
    color: red;
    min-width: 1px;
    min-height: 1px;
    margin: 0px 4px;
}

/* Changes for composite TVs */

.composite headerbar entry {
      font-size: 24px;
  }
